package fr.sanchezm.attestationsCovid19.data.db.dao

import android.util.Log
import androidx.lifecycle.LiveData
import androidx.lifecycle.MutableLiveData
import com.google.gson.Gson
import com.google.gson.reflect.TypeToken
import fr.sanchezm.attestationsCovid19.data.db.entity.Attestation
import fr.sanchezm.attestationsCovid19.data.repository.ConfigRepository
import fr.sanchezm.attestationsCovid19.utilities.PdfUtils
import java.io.File

class AttestationDao(
    private val savePath: String,
    private val filesPath: String,
    private val configRepository: ConfigRepository
) {

    private val _attestations = MutableLiveData<ArrayList<Attestation>>()
    private val fileName = "attestation.db"
    private val type = object : TypeToken<ArrayList<Attestation>>() {}.type

    @Suppress("UNCHECKED_CAST")
    fun getAttestations(): LiveData<List<Attestation>> =
        _attestations as LiveData<List<Attestation>>

    fun getAttestation(id: Int): Attestation? =
        _attestations.value?.elementAt(id)

    fun addAttestation(attestation: Attestation) {
        _attestations.value?.add(attestation)
        _attestations.value = _attestations.value
        Log.d(TAG, "Add Attestation : $attestation")
        save()
    }

    fun getAttestation(createAt: Long): Attestation? {
        _attestations.value?.forEach {
            if (it.createAt == createAt)
                return@getAttestation it
        }
        return null
    }

    fun deleteAttestation(attestation: Attestation) {
        deleteFile(attestation)
        delete(attestation)
    }

    init {
        load()
    }

    private fun delete(attestation: Attestation) {
        _attestations.value?.remove(attestation)
    }

    private fun deleteFile(attestation: Attestation) {
        File(PdfUtils.getPath(filesPath, attestation.createAt)).also {
            if (it.exists())
                it.delete()
        }
        _attestations.value = _attestations.value

    }

    private fun load() {
        val file = File("$savePath/$fileName")
        val removeList = ArrayList<Attestation>()

        if (file.exists()) {
            _attestations.value = Gson().fromJson(file.readText(Charsets.UTF_8), type)
        } else {
            _attestations.value = ArrayList()
        }
        _attestations.value?.forEach {
            val filePath = PdfUtils.getPath(filesPath, it.createAt)

            if (!File(filePath).exists()) {
                removeList.add(it)
            }
        }.also { _attestations.value?.removeAll(removeList) }
    }

    private fun save() {
        Log.d(TAG, "Save : ${Gson().toJson(_attestations.value)}")
        File("$savePath/$fileName").writeText(Gson().toJson(_attestations.value))
    }

    companion object {
        private val TAG = AttestationDao::class.simpleName
    }
}