package fr.sanchezm.attestationsCovid19.data.db.dao

import android.util.Log
import androidx.lifecycle.LiveData
import androidx.lifecycle.MutableLiveData
import com.google.gson.Gson
import com.google.gson.reflect.TypeToken
import fr.sanchezm.attestationsCovid19.BuildConfig
import fr.sanchezm.attestationsCovid19.data.db.entity.Profile
import fr.sanchezm.attestationsCovid19.data.repository.ConfigRepository
import fr.sanchezm.attestationsCovid19.utilities.DbMigration
import java.io.File

class ProfileDao(private val path: String, private val configRepository: ConfigRepository) {

    private val _profile = MutableLiveData<ArrayList<Profile>>()
    private val fileName = "profile.db"
    private val type = object : TypeToken<ArrayList<Profile>>() {}.type

    @Suppress("UNCHECKED_CAST")
    fun getProfiles(): LiveData<List<Profile>> =
        _profile as LiveData<List<Profile>>

    fun getProfile(id: Int): Profile? =
        _profile.value?.get(id)

    fun getProfileId(profile: Profile): Int? =
        _profile.value?.indexOf(profile)

    fun addProfile(profile: Profile): Int {
        _profile.value?.add(profile)
        _profile.value = _profile.value
        save()

        return _profile.value?.indexOf(profile)!!
    }

    init {
        val currentDbVersion = configRepository.getConfig().value!!.dbVersion
        val appDbVersion = BuildConfig.DB_VERSION

        if (currentDbVersion != appDbVersion) {
            if (currentDbVersion == 1 && appDbVersion == 2) {
                _profile.value = DbMigration.migrateProfileFrom1To2("$path/$fileName")
                save()
                configRepository.getConfig().value!!.let {
                    it.dbVersion = appDbVersion
                    configRepository.updateConfig(it)
                }
                Log.d(TAG, "Db version update from $currentDbVersion to $appDbVersion")
            } else {
                File("$path/$fileName").delete()
                load()
            }
        } else {
            load()
        }
    }

    private fun load() {
        val file = File("$path/$fileName")

        if (file.exists()) {
            _profile.value = Gson().fromJson(file.readText(Charsets.UTF_8), type)
        } else {
            _profile.value = ArrayList()
        }
    }

    private fun save() {
        File("$path/$fileName").writeText(Gson().toJson(_profile.value))
    }

    companion object {
        private val TAG = ProfileDao::class.simpleName
    }
}
